@@ -15,7 +15,7 @@
 #
 # END HEADER
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
 
 class HypothesisException(Exception):
@@ -50,10 +50,9 @@ class NoSuchExample(HypothesisException):
     unable to find one.
     """
 
-    def __init__(self, condition_string, extra=''):
+    def __init__(self, condition_string, extra=""):
         super(NoSuchExample, self).__init__(
-            'No examples found of condition %s%s' % (
-                condition_string, extra)
+            "No examples found of condition %s%s" % (condition_string, extra)
         )
 
 
@@ -184,15 +183,15 @@ class DeadlineExceeded(HypothesisException):
     """Raised when an individual test body has taken too long to run."""
 
     def __init__(self, runtime, deadline):
-        super(DeadlineExceeded, self).__init__((
-            'Test took %.2fms, which exceeds the deadline of '
-            '%.2fms') % (runtime, deadline))
+        super(DeadlineExceeded, self).__init__(
+            ("Test took %.2fms, which exceeds the deadline of " "%.2fms")
+            % (runtime, deadline)
+        )
         self.runtime = runtime
         self.deadline = deadline
 
 
 class StopTest(BaseException):
-
     def __init__(self, testcounter):
         super(StopTest, self).__init__(repr(testcounter))
         self.testcounter = testcounter
